@@ -90,11 +90,13 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
 {
 	CDPlayerData *cd = data;
 	cdrom_device_status_t stat;
+	int status;
 
 	if(!cd_try_open(cd))
 		return 0;
 
-	if (cdrom_get_status(cd->cdrom_device, &stat) == DISC_NO_ERROR) {
+	status = cdrom_get_status(cd->cdrom_device, &stat);
+	if (status == DISC_NO_ERROR) {
 		switch (stat.audio_status) {
 		case DISC_PLAY:
 			cdrom_pause(cd->cdrom_device);
@@ -110,6 +112,11 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
 				   cd->cdrom_device->track1);
 			break;
 		}
+	} else if(status == DISC_TRAY_OPEN) {
+		cdrom_load(cd->cdrom_device);
+		cdrom_read_track_info(cd->cdrom_device);
+		cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
+			   cd->cdrom_device->track1);
 	}
 
 	return 0;
@@ -154,10 +161,22 @@ cdplayer_next(GtkWidget * w, gpointer data)
 static int 
 cdplayer_eject(GtkWidget * w, gpointer data)
 {
+	cdrom_device_status_t stat;
 	CDPlayerData *cd = data;
 	if(!cd_try_open(cd))
 		return 0;
-	cdrom_eject(cd->cdrom_device);
+	if(cdrom_get_status(cd->cdrom_device, &stat)==DISC_TRAY_OPEN)
+		/*FIXME: if there is no disc, we get TRAY_OPEN even
+		  if the tray is closed, is this a kernel bug?? or
+		  is this the inteded behaviour, we don't support this
+		  on solaris anyway, but unless we have a good way to
+		  find out if the tray is actually open, we can't do this*/
+		/*cdrom_load(cd->cdrom_device);*/
+		/*in the meantime let's just do eject so that at least that
+		  works*/
+		cdrom_eject(cd->cdrom_device);
+	else
+		cdrom_eject(cd->cdrom_device);
 	return 0;
 }
 
@@ -258,6 +277,11 @@ destroy_cdplayer(GtkWidget * widget, void *data)
 	g_free(cd);
 }
 
+void
+cdpanel_realized(GtkWidget *cdpanel, CDPlayerData *cd)
+{
+	cd_panel_update(cdpanel, cd);
+}
 
 static GtkWidget *
 create_cdplayer_widget(GtkWidget *window, char *globcfgpath)
@@ -284,9 +308,10 @@ create_cdplayer_widget(GtkWidget *window, char *globcfgpath)
 
 	gtk_object_set_user_data(GTK_OBJECT(cdpanel), cd);
 	gtk_signal_connect(GTK_OBJECT(cdpanel), "destroy",
-			   (GtkSignalFunc) destroy_cdplayer,
+			   GTK_SIGNAL_FUNC (destroy_cdplayer),
 			   NULL);
-	cd_panel_update(cdpanel, cd);
+	gtk_signal_connect(GTK_OBJECT (cdpanel), "realize",
+			   GTK_SIGNAL_FUNC (cdpanel_realized), cd);
 
 	return cdpanel;
 }
@@ -296,6 +321,9 @@ main(int argc, char **argv)
 {
 	GtkWidget *cdplayer;
 
+	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
+	textdomain(PACKAGE);
+
 	applet_widget_init("cdplayer_applet", VERSION, argc, argv,
 				    NULL, 0, NULL);
 
