@@ -24,6 +24,7 @@ GdkPixmap          *p_1 = NULL, *p_2 = NULL, *p_3 = NULL;
 GdkPixmap          *m_1 = NULL, *m_2 = NULL, *m_3 = NULL;
 
 typedef struct {
+        gint                reverse_order; /*bool*/
 	gint                pager_rows;
 	gint                pager_size; /*bool*/
 	gint                tasks_all; /*bool*/
@@ -39,6 +40,8 @@ typedef struct {
 	gint                show_icons; /*bool*/
 	gint		    show_arrow; /*bool*/
 	gint                fixed_tasklist; /*bool*/
+	gint                pager_w_0, pager_h_0;
+	gint                pager_w_1, pager_h_1;
 } Config;
 
 /*note: no need for defaults as they will be read in*/
@@ -188,7 +191,8 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 {
   static GnomeHelpMenuEntry help_entry = { NULL, "properties" };
   static GtkWidget *prop = NULL;
-  GtkWidget *hbox,*hbox1,*vbox,*vbox1,*frame,*label, *spin, *check, *rb1, *sh;
+  GtkWidget *hbox,*hbox1,*vbox,*vbox1,*frame,*label, *spin, *check, *sh;
+  GtkWidget *rb1, *rb2;
   GtkAdjustment *adj;
   
   if(prop)
@@ -226,6 +230,16 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
   gtk_object_set_data(GTK_OBJECT(check), "prop", prop);
   gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
   
+  check = gtk_check_button_new_with_label(_("Place pagers after tasklist"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), config.reverse_order);
+  gtk_signal_connect(GTK_OBJECT(check), "toggled",
+		     GTK_SIGNAL_FUNC(cb_check), &o_config.reverse_order);
+  gtk_object_set_data(GTK_OBJECT(check), "prop", prop);
+  gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 0);
+  gtk_signal_connect(GTK_OBJECT(sh), "toggled",
+		     GTK_SIGNAL_FUNC(cb_check_enable), check);
+  cb_check_enable(sh,check);
+
   check = gtk_check_button_new_with_label(_("Use small pagers"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), config.pager_size);
   gtk_signal_connect(GTK_OBJECT(check), "toggled",
@@ -247,6 +261,55 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
   gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_rows);
   gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
+  adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.pager_w_0, 1, 
+					    100, 1, 1, 1 );
+  label = gtk_label_new(_("Width of small pagers"));
+  spin = gtk_spin_button_new(adj, 1, 0);
+  hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
+  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),spin,FALSE,FALSE,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_w_0);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
+  adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.pager_h_0, 1, 
+					    100, 1, 1, 1 );
+  label = gtk_label_new(_("Height of small pagers"));
+  spin = gtk_spin_button_new(adj, 1, 0);
+  hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
+  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),spin,FALSE,FALSE,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_h_0);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
+  adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.pager_w_1, 1, 
+					    100, 1, 1, 1 );
+  label = gtk_label_new(_("Width of large pagers"));
+  spin = gtk_spin_button_new(adj, 1, 0);
+  hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
+  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),spin,FALSE,FALSE,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_w_1);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
+  adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.pager_h_1, 1, 
+					    100, 1, 1, 1 );
+  label = gtk_label_new(_("Height of large pagers"));
+  spin = gtk_spin_button_new(adj, 1, 0);
+  hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
+  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(hbox),spin,FALSE,FALSE,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_h_1);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
   gtk_signal_connect(GTK_OBJECT(sh), "toggled",
 		     GTK_SIGNAL_FUNC(cb_check_enable), hbox);
   cb_check_enable(sh,hbox);
@@ -285,6 +348,8 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
   hbox1 = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
   gtk_box_pack_start(GTK_BOX(vbox),hbox1,FALSE,FALSE,0);
   
+  /* Radio buttons for which tasks to show */
+  
   frame = gtk_frame_new(_("Which tasks to show"));
   gtk_box_pack_start(GTK_BOX(hbox1),frame,FALSE,FALSE,0);
   vbox1 = gtk_vbox_new(FALSE,GNOME_PAD_SMALL);
@@ -296,27 +361,29 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 
   rb1 = check = gtk_radio_button_new_with_label(NULL,_("Show all tasks"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check), config.minimized_tasks);
-  gtk_signal_connect(GTK_OBJECT(check), "toggled",
-		     GTK_SIGNAL_FUNC(cb_check), &o_config.minimized_tasks);
   gtk_object_set_data(GTK_OBJECT(check), "prop", prop);
   gtk_box_pack_start(GTK_BOX(vbox1),check,FALSE,FALSE,0);
 
-  check = gtk_radio_button_new_with_label(
-			  gtk_radio_button_group (GTK_RADIO_BUTTON (rb1)),
+  check = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (rb1),
 			  _("Show normal tasks only"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check),
 				!config.minimized_tasks && !config.minimized_tasks_only);
   gtk_box_pack_start(GTK_BOX(vbox1),check,FALSE,FALSE,0);
   
-  check = gtk_radio_button_new_with_label(
-			  gtk_radio_button_group (GTK_RADIO_BUTTON (rb1)),
+  rb2 = check = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (rb1),
 			  _("Show minimized tasks only"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check), config.minimized_tasks_only);
-  gtk_signal_connect(GTK_OBJECT(check), "toggled",
-		     GTK_SIGNAL_FUNC(cb_check), &o_config.minimized_tasks_only);
   gtk_object_set_data(GTK_OBJECT(check), "prop", prop);
   gtk_box_pack_start(GTK_BOX(vbox1),check,FALSE,FALSE,0);
 
+  /* We connect these signals afterwards, so that when we set the
+   * states above, we don't trigger these callbacks.
+   */
+  gtk_signal_connect(GTK_OBJECT(rb1), "toggled",
+		     GTK_SIGNAL_FUNC(cb_check), &o_config.minimized_tasks);
+  gtk_signal_connect(GTK_OBJECT(rb2), "toggled",
+		     GTK_SIGNAL_FUNC(cb_check), &o_config.minimized_tasks_only);
+
   check = gtk_check_button_new_with_label(_("Show all tasks on all desktops"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), config.tasks_all);
   gtk_signal_connect(GTK_OBJECT(check), "toggled",
@@ -1352,7 +1419,12 @@ cb_applet_save_session(GtkWidget *w,
   gnome_config_set_int("stuff/show_pager", config.show_pager);
   gnome_config_set_int("stuff/show_icons", config.show_icons);
   gnome_config_set_int("stuff/show_arrow", config.show_arrow);
+  gnome_config_set_bool("stuff/reverse_order", config.reverse_order);
   gnome_config_set_int("stuff/fixed_tasklist", config.fixed_tasklist);
+  gnome_config_set_int("stuff/pager_w_0", config.pager_w_0);
+  gnome_config_set_int("stuff/pager_h_0", config.pager_h_0);
+  gnome_config_set_int("stuff/pager_w_1", config.pager_w_1);
+  gnome_config_set_int("stuff/pager_h_1", config.pager_h_1);
   gnome_config_pop_prefix();
   gnome_config_sync();
   gnome_config_drop_all();
@@ -1433,6 +1505,11 @@ main(int argc, char *argv[])
   config.show_pager = gnome_config_get_int("gnome_pager/stuff/show_pager=1");
   config.show_icons = gnome_config_get_int("gnome_pager/stuff/show_icons=1");
   config.show_arrow = gnome_config_get_int("gnome_pager/stuff/show_arrow=1");
+  config.pager_w_0 = gnome_config_get_int("gnome_pager/stuff/pager_w_0=31");
+  config.pager_h_0 = gnome_config_get_int("gnome_pager/stuff/pager_h_0=22");
+  config.pager_w_1 = gnome_config_get_int("gnome_pager/stuff/pager_w_1=62");
+  config.pager_h_1 = gnome_config_get_int("gnome_pager/stuff/pager_h_1=44");
+
   /*make sure these are not done next time*/
   gnome_config_clean_file("gnome_pager");
 
@@ -1509,6 +1586,7 @@ main(int argc, char *argv[])
   config.show_icons = gnome_config_get_int("stuff/show_icons=1");
   config.show_arrow = gnome_config_get_int("stuff/show_arrow=1");
   config.fixed_tasklist = gnome_config_get_int("stuff/fixed_tasklist=0");
+  config.reverse_order = gnome_config_get_bool("stuff/reverse_order=false");
 
   gdk_error_warnings = 0;  
   get_desktop_names();
@@ -1735,8 +1813,13 @@ GList  *get_task_stacking(gint desk)
 	    t = (Task *)p2->data;
 	    
 	    if (t->frame == wl[count]) {
-	      retval = g_list_prepend(retval, (gpointer) t);
 	      tasks_on_desk=g_list_remove_link(tasks_on_desk, p2);
+	      
+	      if (retval)
+		      retval->prev = p2;
+	      p2->next = retval;
+	      p2->prev = NULL;
+	      retval = p2;
 	      break;
 	    }
 	  }
@@ -1781,6 +1864,24 @@ actual_redraw(gpointer data)
   /* FIXME: sel is currently always zero */
   sel = GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "select"));
 
+  if (config.pager_size)
+    {
+      if (widget->allocation.width != (area_w * config.pager_w_0))
+	gtk_widget_set_usize(widget, (area_w * config.pager_w_0), 
+			 widget->allocation.height);
+      if (widget->allocation.height != (area_h * config.pager_h_0))
+	gtk_widget_set_usize(widget, widget->allocation.width, 
+			 (area_h * config.pager_h_0));
+    }
+  else
+    {
+      if (widget->allocation.width != (area_w * config.pager_w_1))
+	gtk_widget_set_usize(widget, (area_w * config.pager_w_1), 
+			 widget->allocation.height);
+      if (widget->allocation.height != (area_h * config.pager_h_1))
+	gtk_widget_set_usize(widget, widget->allocation.width, 
+			 (area_h * config.pager_h_1));
+    }
   w = widget->allocation.width - 4;
   h = widget->allocation.height - 4;
   s = widget->style;
@@ -1888,6 +1989,7 @@ make_desktop_pane(gint desktop, gint width, gint height)
   else
     height = (height * area_h) / area_w;
   gtk_drawing_area_size(GTK_DRAWING_AREA(area), width, height);
+  gtk_widget_set_usize(area, width, height);
   gtk_object_set_data(GTK_OBJECT(area), "desktop", GINT_TO_POINTER (desktop));
   gtk_object_set_data(GTK_OBJECT(area), "select", GINT_TO_POINTER (0));
   gtk_widget_set_events(area, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -1992,7 +2094,11 @@ init_applet_gui(int horizontal)
   align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
   if (config.show_pager)
     gtk_widget_show(align);
-  gtk_box_pack_start(GTK_BOX(main_box), align, FALSE, FALSE, 0);
+
+  if (config.reverse_order)
+     gtk_box_pack_end(GTK_BOX(main_box), align, FALSE, FALSE, 0);
+  else
+     gtk_box_pack_start(GTK_BOX(main_box), align, FALSE, FALSE, 0);
   
   frame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
@@ -2008,9 +2114,9 @@ init_applet_gui(int horizontal)
   for (i = 0; i < num_desk; i++)
     {
       if (config.pager_size)
-	desk = make_desktop_pane(i, PAGER_W_0, PAGER_H_0);
+	desk = make_desktop_pane(i, config.pager_w_0, config.pager_h_0);
       else
-	desk = make_desktop_pane(i, PAGER_W_1, PAGER_H_1);
+	desk = make_desktop_pane(i, config.pager_w_1, config.pager_h_1);
       desk_widget[i] = desk;
       gtk_widget_show(desk);
       gtk_table_attach_defaults(GTK_TABLE(table), desk,
@@ -2064,11 +2170,17 @@ init_applet_gui(int horizontal)
      gtk_widget_show(arrow);
      gtk_container_add(GTK_CONTAINER(arrow_button), arrow);
 
-     gtk_box_pack_start(GTK_BOX(main_box), box, FALSE, FALSE, 0);
+     if (config.reverse_order)
+       gtk_box_pack_end(GTK_BOX(main_box), box, FALSE, FALSE, 0);
+     else
+       gtk_box_pack_start(GTK_BOX(main_box), box, FALSE, FALSE, 0);
     }
   
   frame = gtk_frame_new(NULL);
-  gtk_box_pack_start(GTK_BOX(main_box), frame, FALSE, FALSE, 0);
+  if (config.reverse_order)
+    gtk_box_pack_end(GTK_BOX(main_box), frame, FALSE, FALSE, 0);
+  else
+    gtk_box_pack_start(GTK_BOX(main_box), frame, FALSE, FALSE, 0);
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
   if (config.show_tasks)
     gtk_widget_show(frame);
@@ -2190,6 +2302,15 @@ cb_nuke(GtkWidget * widget, Task *t)
    client_win_kill(t);
 }
 
+void
+cb_stick(GtkWidget * widget, Task *t)
+{
+  if (t->sticky)
+    client_win_unstick(t);
+  else
+    client_win_stick(t);
+}
+
 void
 emtpy_task_widgets(void)
 {
@@ -2413,6 +2534,12 @@ popup_button_pressed (GtkWidget *widget, GdkEventButton *event, Task *t)
 		gtk_signal_connect(GTK_OBJECT(item),"activate",
 				   GTK_SIGNAL_FUNC(cb_shade),t);
 
+		item = gtk_menu_item_new_with_label(_("Stick / Unstick"));
+		gtk_widget_show(item);
+		gtk_menu_append(GTK_MENU(popup),item);
+		gtk_signal_connect(GTK_OBJECT(item),"activate",
+				   GTK_SIGNAL_FUNC(cb_stick),t);
+
 		item = gtk_menu_item_new_with_label(_("Close"));
 		gtk_widget_show(item);
 		gtk_menu_append(GTK_MENU(popup),item);
