@@ -16,6 +16,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include <gdk/gdkx.h>
 #include <gdk/gdkprivate.h>
@@ -72,6 +73,9 @@ struct terminal_config {
 	int scroll_out       :1;       		/* Scroll on output? */
 	int swap_keys        :1;       		/* Swap DEL/Backspace? */
 	int login_by_default :1;                /* do --login as default */
+#ifdef ZVT_BACKGROUND_SCROLL
+	int scroll_background:1; 		/* background will scroll */
+#endif
 	int color_type; 			/* The color mode */
 	enum color_set_enum color_set;
 	char *font; 				/* Font used by the terminals */
@@ -86,9 +90,9 @@ struct terminal_config {
 	int transparent;
 	int shaded;
 	int background_pixmap;
-	char * pixmap_file;
-        char * window_title;                    /* the window title */
-	char * wordclass;			/* select-by-word character class */
+	char *pixmap_file;
+        char *window_title;                     /* the window title */
+	char *wordclass;			/* select-by-word character class */
 } ;
 
 /* Initial command */
@@ -115,6 +119,9 @@ typedef struct {
 	GtkWidget *pixmap_entry;
 	GtkWidget *transparent_checkbox;
 	GtkWidget *shaded_checkbox;
+#ifdef ZVT_BACKGROUND_SCROLL
+	GtkWidget *pixmap_scrollable_checkbox;
+#endif
 	GtkWidget *menubar_checkbox;
         GtkWidget *bell_checkbox;
 	GtkWidget *font_entry;
@@ -221,14 +228,15 @@ gnome_term_set_font (ZvtTerm *term, char *font_name)
 	GdkFont *font;
 
 	font = gdk_font_load (font_name);
-	if (font)
+	if (font) {
 #ifdef ZVT_TERM_EMBOLDEN_SUPPORT
 		if (zvt_term_get_capabilities(term) & ZVT_TERM_EMBOLDEN_SUPPORT)
 			zvt_term_set_fonts  (term, font, 0);
 		else
 #endif
 			zvt_term_set_fonts  (term, font, font);
-	
+	}
+
 	s = gtk_object_get_user_data (GTK_OBJECT (term));
 	if (s)
 		g_free (s);
@@ -414,6 +422,9 @@ load_config (char *class)
 
 	cfg->login_by_default = gnome_config_get_bool ("login_by_default=0");
 
+#ifdef ZVT_BACKGROUND_SCROLL
+	cfg->scroll_background = gnome_config_get_bool ("scroll_background=0");
+#endif
 	/* Default colors in the case the color set is the custom one */
 	fore_color = gnome_config_get_string ("foreground=gray");
 	back_color = gnome_config_get_string ("background=black");
@@ -467,6 +478,11 @@ gather_changes (ZvtTerm *term)
 	newcfg->wordclass = g_strdup (gtk_entry_get_text (GTK_ENTRY (prefs->wordclass_entry)));
 	newcfg->scrollbar_position = option_menu_get_history (GTK_OPTION_MENU (prefs->scrollbar));
 
+#ifdef ZVT_BACKGROUND_SCROLL
+	newcfg->scroll_background = GTK_TOGGLE_BUTTON (prefs->pixmap_scrollable_checkbox)->active
+		&& newcfg->background_pixmap;
+#endif
+
 	g_free (newcfg->font);
 	newcfg->font  = g_strdup (gtk_entry_get_text (GTK_ENTRY (prefs->font_entry)));
 
@@ -538,11 +554,18 @@ apply_changes (ZvtTerm *term, struct terminal_config *newcfg)
 	zvt_term_set_scrollback (term, cfg->scrollback);
 	zvt_term_set_del_key_swap (term, cfg->swap_keys);
 
-	if (zvt_pixmap_support && cfg->background_pixmap)
+	if (zvt_pixmap_support && cfg->background_pixmap) {
+		int flags;
+#ifdef ZVT_BACKGROUND_SCROLL
+		flags = cfg->shaded?ZVT_BACKGROUND_SHADED:0;
+		flags |= cfg->scroll_background?ZVT_BACKGROUND_SCROLL:0;
+#else
+		flags = cfg->shaded;
+#endif
 		zvt_term_set_background (term,
 					 cfg->pixmap_file,
-					 cfg->transparent, cfg->shaded);
-	else if (zvt_pixmap_support && cfg->transparent)
+					 cfg->transparent, flags);
+	} else if (zvt_pixmap_support && cfg->transparent)
 		zvt_term_set_background (term,
 					 NULL,
 					 cfg->transparent, cfg->shaded);
@@ -683,6 +706,10 @@ check_image_options_sensitivity (preferences_t *prefs)
 	gtk_widget_set_sensitive (prefs->pixmap_file_entry, has_background);
 	gtk_widget_set_sensitive (prefs->shaded_checkbox,
 				  has_background || has_transparency);
+#ifdef ZVT_BACKGROUND_SCROLL
+	gtk_widget_set_sensitive (prefs->pixmap_scrollable_checkbox,
+				  has_background);
+#endif
 }
 
 /*
@@ -892,6 +919,9 @@ save_preferences (GtkWidget *widget, ZvtTerm *term,
 	gnome_config_set_bool   ("scrollonoutput", cfg->scroll_out);
 	gnome_config_set_bool   ("transparent", cfg->transparent);
 	gnome_config_set_bool   ("shaded", cfg->shaded);
+#ifdef ZVT_BACKGROUND_SCROLL
+	gnome_config_set_bool   ("scroll_background", cfg->scroll_background);
+#endif
 	gnome_config_set_bool   ("background_pixmap", cfg->background_pixmap);
 	gnome_config_set_string ("pixmap_file", cfg->pixmap_file);
 	gnome_config_sync ();
@@ -1097,7 +1127,6 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	gtk_table_set_row_spacing (GTK_TABLE (subtable), 1, GNOME_PAD_SMALL);
 	
 	/* Background pixmap filename */
-
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_table_attach (GTK_TABLE (subtable), hbox,
 			  0, 2, 2, 3,
@@ -1116,6 +1145,18 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_box_pack_start (GTK_BOX (hbox), prefs->pixmap_file_entry, TRUE, TRUE, 0);
 
+	/* Scrollable pixmap */
+#ifdef ZVT_BACKGROUND_SCROLL
+	prefs->pixmap_scrollable_checkbox = gtk_check_button_new_with_label (_("Background pixmap scrolls"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->pixmap_scrollable_checkbox),
+				     cfg->scroll_background ? 1 : 0);
+	gtk_signal_connect (GTK_OBJECT (prefs->pixmap_scrollable_checkbox), "toggled",
+			    GTK_SIGNAL_FUNC (prop_changed), prefs);
+	gtk_table_attach (GTK_TABLE (subtable), prefs->pixmap_scrollable_checkbox,
+			  0, 1, 3, 4,
+			  GTK_FILL, 0, 0, 0);
+#endif
+
 	/* Transparency */
 	r = prefs->transparent_checkbox =
 		gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (r),
@@ -1125,7 +1166,11 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	gtk_signal_connect (GTK_OBJECT (prefs->transparent_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (subtable), r,
+#ifdef ZVT_BACKGROUND_SCROLL
+			  0, 1, 4, 5,
+#else
 			  0, 1, 3, 4,
+#endif
 			  GTK_FILL, 0, 0, 0);
 
 	/* We pack this box into the the table so the second column
@@ -1134,11 +1179,15 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	 */
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_table_attach (GTK_TABLE (subtable), hbox,
+#ifdef ZVT_BACKGROUND_SCROLL
+			  1, 2, 5, 6,
+#else
 			  1, 2, 4, 5,
+#endif
 			  GTK_FILL | GTK_EXPAND, 0, 0, 0);
 
 	/* Shaded */
-	prefs->shaded_checkbox = gtk_check_button_new_with_label (_("Background should be shaded (slow)"));
+	prefs->shaded_checkbox = gtk_check_button_new_with_label (_("Background should be shaded"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->shaded_checkbox),
 				     term->shaded ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->shaded_checkbox), "toggled",
@@ -1568,7 +1617,7 @@ button_press (GtkWidget *widget, GdkEventButton *event, ZvtTerm *term)
 	GnomeUIInfo *uiinfo;
 
 	if (event->button != 3
-	    || !(event->state & GDK_CONTROL_MASK)
+	    || (!(event->state & GDK_CONTROL_MASK) && term->vx->selected)
 	    || (term->vx->vt.mode & VTMODE_SEND_MOUSE))
 		return FALSE;
 
@@ -1602,13 +1651,11 @@ size_allocate (GtkWidget *widget)
 	app = GNOME_APP (gtk_widget_get_toplevel (GTK_WIDGET (term)));
 	g_assert (app != NULL);
 
-	/* eek, this isn't right, but it kinda works */
 #define PADDING 2
 	sizehints.base_width = 
 	  (GTK_WIDGET (app)->allocation.width) +
-	  (GTK_WIDGET (term)->style->klass->xthickness * 4) -
-	  (GTK_WIDGET (term)->allocation.width) +
-		PADDING;
+	  (GTK_WIDGET (term)->style->klass->xthickness * 2) -
+	  (GTK_WIDGET (term)->allocation.width) + PADDING;
 
 	sizehints.base_height =
 	  (GTK_WIDGET (app)->allocation.height) +
@@ -1646,6 +1693,36 @@ term_change_pos(GtkWidget *widget)
 		gtk_widget_queue_draw(widget);
 }
 
+/* do this because there have been only 5 bug reports about this */
+static void
+show_pty_error_dialog (int errcode)
+{
+	char *tmpmsg, *errmsg;
+	GtkWidget *dialog;
+
+	perror ("Error: unable to fork");
+
+	tmpmsg = errcode ? 
+		g_strdup_printf(_("The error was: %s"), g_strerror(errcode)) : 
+		_("If you are using Linux 2.2.x with glibc 2.1.x, this\n"
+		  "is probably due to incorrectly setup Unix98 ptys.\n\n"
+		  "Please read linux/Documentation/Changes for how to\n"
+		  "set them up correctly.");
+	
+	errmsg =  g_strdup_printf(_("There has been an error while "
+				    "trying to log in.\n\n%s"), tmpmsg);
+	
+	
+	dialog = gnome_message_box_new (errmsg,
+					GNOME_MESSAGE_BOX_ERROR,
+					GNOME_STOCK_BUTTON_OK, NULL);
+							
+	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+
+	if (errcode) g_free(tmpmsg);
+	g_free(errmsg);
+}
+
 GtkWidget *
 new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 {
@@ -1657,6 +1734,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	char *shell, *name;
 	int i = 0;
 	struct terminal_config *cfg;
+	int width, height;
 
 	/* FIXME: is seems like a lot of this stuff should be done by apply_changes instead */
 
@@ -1707,7 +1785,28 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	terminals = g_list_append (terminals, app);
 
 	/* Setup the Zvt widget */
-	term = ZVT_TERM (zvt_term_new_with_size (80, 25));
+
+	/*
+	 * Handle geometry specification; height and width are in
+	 * terminal rows and columns
+	 */
+	width=80;
+	height=24;
+	if (geometry){
+		int xpos, ypos;
+		
+		gnome_parse_geometry (geometry, &xpos, &ypos, &width, &height);
+		if (xpos != -1 && ypos != -1)
+			gtk_widget_set_uposition (GTK_WIDGET (app), xpos, ypos);
+		if (width == -1 || height == -1) {
+			width=80;
+			height=24;
+		}
+		/* Only the first window gets --geometry treatment for now */
+		geometry = NULL;
+	}
+
+	term = (ZvtTerm *)zvt_term_new_with_size(width, height);
 
 	if ((zvt_term_get_capabilities (term) & ZVT_TERM_PIXMAP_SUPPORT) != 0){
 		zvt_pixmap_support = TRUE;
@@ -1748,8 +1847,8 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	/* Decorations */
 	hbox = gtk_hbox_new (0, 0);
 
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
-	gtk_box_set_spacing (GTK_BOX (hbox), 3);
+/*	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2); */
+       	gtk_box_set_spacing (GTK_BOX (hbox), 3);
 
 	gtk_widget_show (hbox);
 	get_shell_name (&shell, &name, cfg->invoke_as_login_shell);
@@ -1770,28 +1869,20 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	
 	gnome_app_set_contents (GNOME_APP (app), hbox);
 
-	/*
-	 * Handle geometry specification; height and width are in
-	 * terminal rows and columns
-	 */
-	if (geometry){
-		int xpos, ypos, width, height;
-		
-		gnome_parse_geometry (geometry, &xpos, &ypos, &width, &height);
-		if (xpos != -1 && ypos != -1)
-			gtk_widget_set_uposition (GTK_WIDGET (app), xpos, ypos);
-		if (width != -1 && height != -1)
-		        zvt_term_set_size (ZVT_TERM (term), width, height);
-		
-		/* Only the first window gets --geometry treatment for now */
-		geometry = NULL;
-	}
 	configure_term_dnd (term);
 
-	if (zvt_pixmap_support && cfg->background_pixmap)
-		zvt_term_set_background (term, cfg->pixmap_file,
-					 cfg->transparent, cfg->shaded);
-	else if (zvt_pixmap_support && cfg->transparent)
+	if (zvt_pixmap_support && cfg->background_pixmap) {
+		int flags;
+#ifdef ZVT_BACKGROUND_SCROLL
+		flags = cfg->shaded?ZVT_BACKGROUND_SHADED:0;
+		flags |= cfg->scroll_background?ZVT_BACKGROUND_SCROLL:0;
+#else
+		flags = cfg->shaded;
+#endif
+		zvt_term_set_background (term,
+					 cfg->pixmap_file,
+					 cfg->transparent, flags);
+	} else if (zvt_pixmap_support && cfg->transparent)
 		zvt_term_set_background (term, NULL,
 					 cfg->transparent, cfg->shaded);
 	else
@@ -1813,9 +1904,13 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	gdk_window_set_hints (((GtkWidget *)app)->window,
 			      0, 0, 50, 50, 0, 0, GDK_HINT_MIN_SIZE);
 
+	XSync(GDK_DISPLAY(), False);
+
+	errno = 0;
 	switch (zvt_term_forkpty (term, update_records)){
 	case -1:
-		perror ("Error: unable to fork");
+		show_pty_error_dialog(errno);
+		/* should we exit maybe? */
 		return NULL;
 		
 	case 0: {
@@ -1824,7 +1919,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 		for (i = 3; i < open_max; i++)
 			fcntl (i, F_SETFD, 1);
 		
-		sprintf (buffer, "WINDOWID=%d",(int) ((GdkWindowPrivate *)app->window)->xwindow);
+		sprintf (buffer, "WINDOWID=%d",(int) GDK_WINDOW_XWINDOW(GTK_WIDGET(term)->window));
 		env_copy [winid_pos] = buffer;
 		if (cmd){
 			environ = env_copy;
