@@ -171,6 +171,8 @@ write_session (void)
   int i = 0;
   GSList *list;
 
+  if (choosing)
+    return;
 
   delete_session (session_name);
 
@@ -248,15 +250,23 @@ read_one_client (Client *client)
 {
   int i, j;
   gboolean def;
+  gchar* id;
 
-  client->id = gnome_config_get_string ("id");
+  client->id = NULL;
   client->properties = NULL;
   client->priority = 50;
   client->handle = command_handle_new ((gpointer)client);
+  client->warning = FALSE;
   client->get_prop_replies = NULL;
   client->get_prop_requests = 0;
   client->command_data = NULL;
 
+  id = gnome_config_get_string ("id");
+  if (id)
+    {
+      client->id = strdup(id);
+      g_free (id);
+    }
   /* Read each property that we save.  */
   for (i = 0; i < NUM_PROPERTIES; ++i)
     {
@@ -349,7 +359,7 @@ read_clients (const char* file, const char *session, MatchRule match_rule)
 
   for (i = 0; i < num_clients; ++i)
     {
-      Client *client = (Client*)malloc (sizeof(Client));
+      Client *client = (Client*)calloc (1, sizeof(Client));
 
       g_snprintf (prefix, sizeof(prefix), "%s%s/%d,", file, session, i);
 
@@ -383,12 +393,17 @@ set_session_name (const char *name)
     {
       /* FIXME: generate a new unique name when the requested one is locked */
       /* FIXME: establish lock on the name */
-      
-      gnome_config_push_prefix (GSM_CONFIG_PREFIX);
-      gnome_config_set_string (CURRENT_SESSION_KEY, name);
-      gnome_config_pop_prefix ();
-      gnome_config_sync ();
-      
+
+      if (trashing)
+	name = TRASH_SESSION;
+
+      if (! choosing && !trashing)
+	{
+	  gnome_config_push_prefix (GSM_CONFIG_PREFIX);
+	  gnome_config_set_string (CURRENT_SESSION_KEY, name);
+	  gnome_config_pop_prefix ();
+	  gnome_config_sync ();
+	}
       if (session_name)
 	unlock_session ();
       session_name = strdup (name);
@@ -422,7 +437,7 @@ read_session (const char *name)
   session->name   = g_strdup (name);
   session->handle = command_handle_new ((gpointer)session);
  
-  try_last = try_def = (session_name == NULL);
+  try_last = try_def = (failsafe || session_name == NULL);
 
   if (try_last)
     {
@@ -434,7 +449,7 @@ read_session (const char *name)
 
   while (name)
     {
-      if ((list = read_clients (CONFIG_PREFIX, name, MATCH_ID)))
+      if (!failsafe && (list = read_clients (CONFIG_PREFIX, name, MATCH_ID)))
 	break;
 	  
       if ((list = read_clients (DEFAULT_CONFIG_PREFIX, name, MATCH_FAKE_ID)))
